#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h> 
#include <math.h>

#define MAG_LIMIT (9.00)
#define HOURS_TO_RADIANS (M_PI/12.0)
#define DEGREES_TO_RADIANS (M_PI/180.0)

#define TRUE  (1)
#define FALSE (0)

typedef struct __attribute__((packed)) starEntry {
  int number;
  double rAJ2000;
  double decJ2000;
  float parallax;
  float muRA;
  float muDec;
  short mag;
  short color;
} starEntry;

/*
  Read a line of text from the config file, and strip comments (flagged by #).
*/
int getLine(int fD, char *buffer, int *eOF)
{
  char inChar = (char)0;
  int count = 0;
  int sawComment = FALSE;
  int foundSomething = FALSE;

  buffer[0] = (char)0;
  while ((!(*eOF)) && (inChar != '\n') && (count < 132)) {
    int nChar;

    nChar = read(fD, &inChar, 1);
    if (nChar > 0) {
      foundSomething = TRUE;
      if (inChar == '#')
        sawComment = TRUE;
      if (!sawComment)
        buffer[count++] = inChar;
    } else {
      *eOF = TRUE;
    }
  }
  if (foundSomething) {
    if (count > 0)
      buffer[count-1] = (char)0;
    return(TRUE);
  } else
    return(FALSE);
}

main()
{
  int nProcessed = 0;
  int outFD;
  int inFD;
  int eof = FALSE;
  char inLine[1000];

  inFD = open("hip.txt", O_RDONLY);
  outFD = open("./stars.dat", O_WRONLY | O_CREAT);
  while (!eof) {
    int negZero;
    int nRead, hipNum, rAHH, rAMM, decDD, decMM;
    double rASS, decSS, mag, par, muRA, muDec, color;

    getLine(inFD, &inLine[0], &eof);
    nRead = sscanf(inLine,
		   "%d %d %d %lf %d %d %lf %lf %lf %lf %lf %lf",
		   &hipNum,
		   &rAHH, &rAMM, &rASS,
		   &decDD, &decMM, &decSS,
		   &mag, &par, &muRA, &muDec, &color);
    if ((nRead == 12) && (mag <= MAG_LIMIT)) {
      starEntry entry;

      entry.number = hipNum;
      entry.rAJ2000 = (((double)rAHH) + ((double)rAMM)/60.0 +
		       rASS/3600.0) * HOURS_TO_RADIANS;
      if (strstr(inLine, "-00 ") != NULL) {
	negZero = TRUE;
	printf("Neg zero line: \"%s\"\n", inLine);
      } else
	negZero = FALSE;
      if ((decDD < 0) || negZero) {
	decMM *= -1.0;
	decSS *= -1.0;
      }
      entry.decJ2000 = (((double)decDD) + ((double)decMM)/60.0 +
		       decSS/3600.0) * DEGREES_TO_RADIANS;
      entry.parallax = (float)par;
      entry.muRA = (float)muRA;
      entry.muDec = (float)muDec;
      entry.mag = (short)(mag*100.0);
      entry.color = (short)(color*1000.0);
      write(outFD, &entry, sizeof(entry));
      nProcessed++;
    }
  }
  printf("Wrote entries for %d stars\n", nProcessed);
}
